@@ -33,7 +33,7 @@
 #include <thai/thwchar.h>
 #include <thai/thbrk.h>
 #include "thbrk-priv.h"
-#include "thbrk-utils.h"
+#include "utils/priv-utils.h"
 #include "brk-maximal.h"
 #include "brk-common.h"
 
@@ -44,13 +44,13 @@ typedef struct _BrkShot {
     TrieState      *dict_state;
     int             str_pos;
     int            *brk_pos;
-    int             n_brk_pos;
+    size_t          n_brk_pos;
     int             cur_brk_pos;
     int             penalty;
 } BrkShot;
 
 static int          brk_shot_init (BrkShot *dst, const BrkShot *src);
-static void         brk_shot_reuse (BrkShot *dst, const BrkShot *src);
+static int          brk_shot_reuse(BrkShot *dst, const BrkShot *src);
 static void         brk_shot_destruct (BrkShot *shot);
 
 /**
@@ -84,13 +84,13 @@ struct _BrkEnv {
  */
 typedef struct {
     int    *brk_pos;
-    int     n_brk_pos;
+    size_t  n_brk_pos;
     int     cur_brk_pos;
     int     str_pos;
     int     penalty;
 } BestBrk;
 
-static BestBrk *    best_brk_new (int n_brk_pos);
+static BestBrk *    best_brk_new (size_t n_brk_pos);
 static void         best_brk_free (BestBrk *best_brk);
 static int          best_brk_contest (BestBrk *best_brk, const BrkShot *shot);
 
@@ -106,7 +106,7 @@ typedef struct {
  *   PRIVATE METHODS DECLARATIONS   *
  *----------------------------------*/
 
-static BrkPool *    brk_root_pool (int pos_size, BrkEnv *env);
+static BrkPool *    brk_root_pool (size_t pos_size, BrkEnv *env);
 static int          brk_maximal_do_impl (const thwchar_t *ws, int len,
                                          const char *brkpos_hints,
                                          int pos[], size_t n, BrkEnv *env);
@@ -371,7 +371,7 @@ recov_done:
 }
 
 static BrkPool *
-brk_root_pool (int pos_size, BrkEnv *env)
+brk_root_pool (size_t pos_size, BrkEnv *env)
 {
     ThBrk      *brk;
     BrkPool    *pool;
@@ -448,19 +448,24 @@ brk_shot_init (BrkShot *dst, const BrkShot *src)
     return 0;
 }
 
-static void
+static int
 brk_shot_reuse (BrkShot *dst, const BrkShot *src)
 {
+    if (dst->n_brk_pos < src->n_brk_pos) {
+        int *p = (int *) realloc (dst->brk_pos,
+                                  src->n_brk_pos * sizeof (int));
+        if (UNLIKELY(!p))
+            return -1;
+        dst->brk_pos = p;
+    }
     trie_state_copy (dst->dict_state, src->dict_state);
     dst->str_pos = src->str_pos;
-    if (dst->n_brk_pos < src->n_brk_pos) {
-        dst->brk_pos = (int *) realloc (dst->brk_pos,
-                                        src->n_brk_pos * sizeof (int));
-    }
     memcpy (dst->brk_pos, src->brk_pos, src->cur_brk_pos * sizeof (int));
     dst->n_brk_pos = src->n_brk_pos;
     dst->cur_brk_pos = src->cur_brk_pos;
     dst->penalty = src->penalty;
+
+    return 0;
 }
 
 static void
@@ -501,16 +506,29 @@ brk_env_free (BrkEnv *env)
 }
 
 static BrkPool *
-brk_pool_node_new (const BrkShot *shot, BrkEnv *env)
+brk_pool_node_new_from_free_list(const BrkShot *shot, BrkEnv *env)
 {
     BrkPool *node;
 
     if (env->free_list) {
         /* reuse old node if possible */
         node = env->free_list;
+        if (UNLIKELY(brk_shot_reuse (&node->shot, shot) != 0))
+            return NULL;
         env->free_list = env->free_list->next;
-        brk_shot_reuse (&node->shot, shot);
-    } else {
+        return node;
+    }
+
+    return NULL;
+}
+
+static BrkPool *
+brk_pool_node_new (const BrkShot *shot, BrkEnv *env)
+{
+    BrkPool *node;
+
+    if (!(node = brk_pool_node_new_from_free_list(shot, env))) {
+        /* no old node or failed to grow old node */
         node = (BrkPool *) malloc (sizeof (BrkPool));
         if (UNLIKELY (!node))
             return NULL;
@@ -624,18 +642,18 @@ brk_pool_delete_node (BrkPool *pool, BrkPool *node, BrkEnv *env)
 }
 
 static BestBrk *
-best_brk_new (int n_brk_pos)
+best_brk_new (size_t n_brk_pos)
 {
     BestBrk *best_brk;
 
-    if (UNLIKELY ((size_t) n_brk_pos > SIZE_MAX / sizeof (int)))
+    if (UNLIKELY (n_brk_pos > SIZE_MAX / sizeof (int)))
         return NULL;
 
     best_brk = (BestBrk *) malloc (sizeof (BestBrk));
     if (UNLIKELY (!best_brk))
         return NULL;
 
-    best_brk->brk_pos = (int *) malloc ((size_t) n_brk_pos * sizeof (int));
+    best_brk->brk_pos = (int *) malloc (n_brk_pos * sizeof (int));
     if (UNLIKELY (!best_brk->brk_pos))
         goto exit1;
     best_brk->n_brk_pos = n_brk_pos;
